%FULLHISTEQ Enhance contrast using histogram equalization.
%   FULLHISTEQ enhances the contrast of images by transforming the
%   values in an intensity image, so that the histogram of the 
%   output image is linear or has a Gaussian shape.
%   
%   B = FULLHISTEQ(A) transforms the intensity image A so that
%   the output image B has a linear histogram. More specificly,
%   the transformation is:
%                HE(g) = nnz(A<=g)/size(A)
%   
%   B = FULLHISTEQ(A,SIGMA) applies a Gaussian histogram with 
%   standard deviation proportional to SIGMA.
%   
%   In this function, the lowest and highest colors are REMOVED, 
%   since those pixels are usually over-exposed or under-exposed 
%   pixels.

% Zvi Devir & Liliya Tridensky
% Sep 2002

function B = fullHisteq(A,trim);

if (nargin<2), trim=0; end;
siz = size(A);
len = prod(siz);

[B,I] = sort(A(:));
clear A
F = B(1:end-1)~=B(2:end);
clear B
F = find([F;1]);

H = F./prod(siz);
if (trim>0),
  % Overcome memory problems with large images...
  % H = erfinv((H-0.5)*erf(trim)*2)/(2*trim)+0.5;
  for i=1:1000000:length(H),
    j = min(i+1000000-1,length(H));
    H(i:j) = erfinv((H(i:j)-0.5)*erf(trim)*2)/(2*trim)+0.5;
  end;
end;

% Handle over-exposure, by removing highest and lowest colors.
H(1) = H(2);
H(end) = H(end-1);
H = (H-H(1))./(H(end)-H(1));

H1 = H(1);
H = H(2:end)-H(1:end-1);
B = zeros([len,1]);
B(1) = H1;
B(F(1:end-1)+1) = H;
clear F H
B = cumsum(B);

B(I) = B;
B = reshape(B,siz);
