function [im, type] = radread( file )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Format inputs.

if (nargin < 1)
  	error('First argument is required');
end

im = []; sensor={}; illuminant={}; type={};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read in data file.

% Open the file.

fid = fopen( file, 'r' );

if (fid == -1)
   error('Could not open file specified.');
end

% Read in relevant header information.

[type,bit] = getFormat(fid);
if isempty(type)
    error('Unknown format');
end

im = getImageData(fid,bit);
fclose( fid );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read in format of file.

function [type,bit] = getFormat(fid)

type=[];
bit=[];

supported = {'32-bit_rle_rgbe','32-bit_rle_xyze','64-bit_rle_rgbe','64-bit_rle_xyze'};
TYPES = {'rgbe','xyze','rgbe','xyze'};
BITS = [8 8 16 16];

format = getRecord(fid,'FORMAT');

for k=1:length(supported)
    if ~isempty(findstr(format,supported{k}))
        type=TYPES{k};
        bit = BITS(k);
        return;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extract sensor sensitivity function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extract illumination information

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extract Radiance Record information

function record = getRecord(fid,name)

maxLine = 20;
record =[];

for i=1:maxLine
    line = fgets( fid );
    if (strncmp( line, name,length(name) ) == 1)
        index = findstr(line,'=');
        if ~isempty(index)
            record = line(index+1:end);
            return;
        end
        return;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read in Radiance RGBE data

function im=getImageData(fid,bit)

line=fgets(fid);
while length(line)~=1
    line=fgets(fid);
end

line = fgets( fid );
locY = findstr(line,'-Y');
locX = findstr(line,'+X');

if (sum(size(locY)) == 0 | sum(size(locX)) == 0)
   error('Only support data in the -Y +X direction.');
end

strsiz = [ line((locY+3):(locX-1)) ...
           line((locX+3):(length(line)-1)) ];
numsiz = str2num(strsiz);

% Read in the data.

if bit ==8
    [file_im len] = fread( fid, inf, 'uint8' );
else 
    [file_im len] = fread( fid, inf, 'uint16' );
end

if (len ~= (prod(numsiz)*4))
   error('Incorrect format.  Use the flat format.');
end

file_im = reshape( file_im, 4, prod(numsiz) )';
tmp_im  = (file_im(:,1:3)./2^bit).*repmat( 2.^(file_im(:,4)-2^(bit-1)), 1, 3 );
im      = reshape(tmp_im, numsiz(2), numsiz(1), 3);

im = permute(im,[2 1 3]);
